#define __USE_BSD
#include <math.h>
#include "mrx_alsa.h"

/********************************************************************

NAME
	init_alsa()

SYNOPSIS
        #include "mrx_alsa.h"
	int init_alsa(void);

DESCRIPTION
        ...
ARGUMENTS
	---

RETURN VALUE + ERRORS
        0: ok
        1: error

NOTES
       needs libasound2-dev (lib-alsa-sound)
SEE ALSO
       --- 
Date       Changes
------------------
25.02.2007 under construction

*********************************************************************/

int init_alsa(void)
{
	int err;
	int drate=DRATE;
	snd_pcm_hw_params_t *hw_params;

	if ((err = snd_pcm_open (&capture_handle, "default", SND_PCM_STREAM_CAPTURE, 0)) < 0) 
	{
		fprintf (stderr, "cannot open audio device for read\n", snd_strerror (err));
		return -1;
	}
	if ((err = snd_pcm_hw_params_malloc (&hw_params)) < 0) 
	{
		fprintf (stderr, "cannot allocate hardware parameter structure (%s)\n", snd_strerror (err));
		return -2;
	}
	if ((err = snd_pcm_hw_params_any (capture_handle, hw_params)) < 0) 
	{
		fprintf (stderr, "cannot initialize hardware parameter structure (%s)\n", snd_strerror (err));
		return -3;
	}
	if ((err = snd_pcm_hw_params_set_access (capture_handle, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0) 
	{
		fprintf (stderr, "cannot set access type (%s)\n", snd_strerror (err));
		return -4;
	}
	if ((err = snd_pcm_hw_params_set_format (capture_handle, hw_params, SND_PCM_FORMAT_S16_LE)) < 0)
	{
		fprintf (stderr, "cannot set sample format (%s)\n", snd_strerror (err));
		return -5;
	}
	if ((err = snd_pcm_hw_params_set_rate_near (capture_handle, hw_params, &drate, 0)) < 0) 
	{
		fprintf (stderr, "cannot set sample rate (%s)\n", snd_strerror (err));
		return -6;
	}
	if ((err = snd_pcm_hw_params_set_channels (capture_handle, hw_params, 1)) < 0) 
	{
		fprintf (stderr, "cannot set channel count (%s)\n", snd_strerror (err));
		return -7;
	}
	if ((err = snd_pcm_hw_params (capture_handle, hw_params)) < 0) 
	{
		fprintf (stderr, "cannot set parameters (%s)\n", snd_strerror (err));
		return -8;
	}
	snd_pcm_hw_params_free (hw_params);
	if ((err = snd_pcm_prepare (capture_handle)) < 0) 
	{
		fprintf (stderr, "cannot prepare audio interface for use (%s)\n", snd_strerror (err));
		return -9;
	}
	return 0;
   
}/* init_alsa() */

/********************************************************************

NAME
	getstat_alsa()

SYNOPSIS
        #include "mrx_alsa.h"
	int getstat_alsa(void);

DESCRIPTION
        ...
ARGUMENTS
	---

RETURN VALUE + ERRORS
        0,1: Return Status.
        -1: error: read impossible
NOTES
       needs libasound2-dev (lib-alsa-sound)
SEE ALSO
       --- 
Date       Changes
---------------------------------------------
25.02.2007 under construction
15.10.2008 recover from overrun (EPIPE error) 
*********************************************************************/

int getstat_alsa(void)
{
   int i, rc;
   double detect_result;
   int16_t buf[NUM_OF_FRAMES];
   double dval[NUM_OF_FRAMES];
   while ((rc = snd_pcm_readi (capture_handle, (void *)buf, NUM_OF_FRAMES)) != NUM_OF_FRAMES)
   {
      if(rc==-EPIPE)
      {
         fprintf (stderr, "overrun detected...\n");
         snd_pcm_prepare(capture_handle);
      }
      else
      {
         fprintf (stderr, "read from audio interface failed (%d/%s)\n", rc, snd_strerror (rc));
         return -1;
      }
   }
   for(i=0;i<NUM_OF_FRAMES;++i)
   {
      double disp;
      dval[i]=buf[i]*100.0/32768;          
   }
   
   detect_result=detect_goertzel(dval, NUM_OF_FRAMES, Frequency);
   rc=(detect_result>DetectLevel); 
   return rc;
   
}/* getstat_alsa() */

/********************************************************************

NAME
	detect_goertzel

SYNOPSIS
        #include "mrx_alsa.h"
	detect_goertzel(dval, num_of_frames, freq);

DESCRIPTION
        goertzel filter algorithm
        
ARGUMENTS
	double dval[]......: input buffer 
	int num_of_frames..: no. of values in input buffer
	double freq........: absolute frequency (Hz)

RETURN VALUE + ERRORS
        double: result
        
SEE ALSO
        --- 
Date       Changes
----------------------------------------------
25.02.2007 under construction
21.10.2008 different frequencies made possible

*********************************************************************/

double detect_goertzel(double dval[], int num_of_frames, double freq)
{
   int i;
   double u1,u2,u3,u4,u5,u6;          /* signals within the IIR filter */
   static double ar0,ar1,ar2,br1,br2; /* filter coeff., real  */
   static double ai0,ai1,ai2,bi1,bi2; /* filter coeff., imag. */
   static double oldfreq=-1.0;

   freq /= DRATE; /* relative frequency */
   
   if(freq!=oldfreq)
   {
      oldfreq=freq;
      ar0 = 1.0;  ar1 = -cos(2.0*M_PI*freq);  ar2 =  0.0;
                  br1 = 2*cos(2.0*M_PI*freq); br2 = -1.0;
                  
      ai0 = 0.0;  ai1 = sin(2.0*M_PI*freq);   ai2 = 0.0;
                  bi1 = 2*cos(2.0*M_PI*freq); bi2 = -1.0;
      
   }/* if */

   u1=u2=u4=u5=0.0;
   for(i=0; i<num_of_frames; ++i)
   {      
      u3=ar0*dval[i] + u2;
      u2=ar1*dval[i] + br1*u3 + u1;
      u1=ar2*dval[i] + br2*u3;
      
      u6=ai0*dval[i] + u5;
      u5=ai1*dval[i] + bi1*u6 + u4;
      u4=ai2*dval[i] + bi2*u6;
      
   }/* for */
   
   return sqrt(u3*u3+u6*u6);

}/* detect_goertzel() */

/********************************************************************

NAME
	exit_alsa()

SYNOPSIS
        #include "mrx_alsa.h"
	exit_alsa();

DESCRIPTION
        close all soundcard connection
        
ARGUMENTS
	---

RETURN VALUE
        void
NOTES
       needs libasound2-dev (lib-alsa-sound)
SEE ALSO
       --- 
Date       Changes
------------------
25.02.2007 under construction

*********************************************************************/

void exit_alsa(void)
{
   snd_pcm_close (capture_handle);
   
}/* exit_alsa() */
